<html>
<body>

<h3>Utilisateurs recevant des recaps</h3>

<ul>
  {% for user in users %}
    <li>
      <a href="{{ url_for(".debug_recap_for", uid=user.id) }}">{{ user.full_name }}
        ({{ user.login }})</a>
    </li>
  {% endfor %}
</ul>

</body>
</html>
