<!DOCTYPE html>
<html lang="fr-FR">

<head>
  <title>Lab&Co - Sorbonne Université</title>

  <meta name="viewport"
      content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no">
  <meta name="description" content="">
  <meta name="author" content="Abilian SAS for Sorbonne Université">

  <link rel="shortcut icon" type="image/png" href="{{ app.config['FAVICO_URL'] | string }}"/>
  <link rel="apple-touch-icon" type="image/png" href="{{ app.config['FAVICO_URL'] | string }}"/>

  {% for url in g.assets.css %}
    <link href="{{ url }}" rel="stylesheet" type="text/css">
  {% endfor %}
  {# TODO #}

  <link rel="stylesheet" type="text/css"
      href="https://unpkg.com/vue-multiselect@2.1.6/dist/vue-multiselect.min.css">

  <script>
    var DEBUG = false;
    var CTX = {};
    CTX.version = "{{ g.labster_version }}";
  </script>

</head>

<body class="labster sidebar-mini">

<div id="app-v3"></div>

{% for url in g.assets.js %}
  <script src="{{ url }}"></script>
{% endfor %}

</body>
</html>
