<html lang="fr">
<body>

<p>
  Le {{ now|datetime }},
</p>

<p>
  {{ workflow.actor.full_name }}, contact pour les demandes de
  type {{ demande.type }}, vous confirme que votre demande "{{ demande.nom }}"
  du {{ demande.created_at|datetime }} a été finalisée. n°
  eOTP {{ demande.no_eotp }}, compte ouvert.
</p>


{% if transition.note %}
  <p>
    Merci de prendre connaissance du message suivant:
  </p>

  <blockquote>
    {{ transition.note }}
  </blockquote>
{% endif %}


{#FIXME!!! <p>#}
{#  Si vous souhaitez donner votre avis sur le traitement de cette demande, <a#}
{#    href="{{ url_for("demandes.questionnaire", demande_id=demande.id, _external=True) }}">cliquez#}
{#  ici</a>.#}
{#</p>#}

{% include "emails/_footer.html" %}

</body>
</html>
