from __future__ import annotations

from .fields import HTML, Boolean2Field, BooleanField, DateField, EmailField, \
    FieldSet, IntegerField, ListeAutresDeclarations, ListeContrats, \
    ListeDivulgationsFutures, ListeDivulgationsPassees, \
    ListeLicencesExistantes, ListeMateriels, ListePartenaires, \
    ListePartenairesContactes, Select2Field, StringField, TextAreaField, \
    TriStateField
from .form import Form
