# Import old stuff for now
from __future__ import annotations

from . import config  # noqa
from .financeurs import *  # noqa
from .mapping_dgrtt import *  # noqa
from .notifications import *  # noqa
from .roles import *  # noqa
