# from __future__ import annotations
#
#
# class BaseInMemoryRepository:
#     def __init__(self):
#         self._data = {}
#
#     def clear(self):
#         self._data.clear()
#
#     def is_empty(self):
#         return len(self._data) == 0
from __future__ import annotations
