{# Nouvelle version du mail de récap #}
<html>

<body>

{%- macro porteur(demande) -%}
  {%- if demande.porteur -%}
    {{ demande.porteur.full_name }}
  {%- else -%}
    (pas de porteur identifié à ce stade)
  {%- endif -%}
{%- endmacro -%}

{%- macro button(url, label) -%}
  {# Using https://litmus.com/blog/a-guide-to-bulletproof-buttons-in-email-design #}
  <table width="100%" border="0" cellspacing="0" cellpadding="0">
    <tr>
      <td>
        <table border="0" cellspacing="0" cellpadding="0">
          <tr>
            <td align="center" style="border-radius: 3px;" bgcolor="#3c8dbc"><a
                href="{{ url }}" target="_blank"
                style="
                  font-size: 13px;
                  font-family: Helvetica, Arial, sans-serif;
                  color: #ffffff;
                  text-decoration: none;
                  border-radius: 3px;
                  padding: 10px 15px;
                  border: 1px solid #3c8dbc;
                  display: inline-block;
                ">{{ label }}</a></td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
{%- endmacro -%}

<h2>Le point sur vos demandes Lab&amp;Co</h2>

<hr>
{% for demande in demandes %}
  {% set url = url_for(demande, _external=True) %}
  <h3>
    Demande: <a href="{{ url }}">{{ demande.nom }}</a> (porteur: {{ porteur(demande) }},
  </h3>

  {% if demande.date_soumission %}
    <p>Soumise le: {{ demande.date_soumission|dateformat("medium") }})</p>
  {% else %}
    <p>Créée le: {{ demande.created_at|dateformat("medium") }})</p>
  {% endif %}

  {% set cta = get_cta(demande) %}
  {% if cta %}
    {{ button(url, cta) }}
  {% endif %}

  {% set notifs = get_notifications(demande) %}
  {% if notifs %}
    <h4>Notifications associées</h4>

    <div class="margin-left: 2em;">
      {% for notif in notifs %}
        <h5>Le {{ notif.created_at | datetimeformat("d MMM Y à H:mm") }}:</h5>

        {{ notif.body | safe }}
      {% endfor %}
    </div>
  {% endif %}

  <hr style="margin-top: 20px;">
{% endfor %}

{% if has_tasks %}
  <h3>Toutes vos tâches en attente</h3>

  <p>
    <a href="{{ url_for("main.demandes_a_valider", _external=True) }}">Voir les
      demandes</a>.
  </p>
{% endif %}

<hr>

{% if config.NAME != 'PRODUCTION' %}
  <p>
    <em>Ceci est un mail de test. Nous vous remercions de réaliser l’action
      demandée par ce mail afin de finaliser le test.<br> Si vous avez des
      questions, contactez Marie-Noëlle Poulain par mail <a
          href="mailto:marie-noelle.poulain@upmc.fr">marie-noelle.poulain@upmc.fr</a>
      ou par téléphone au 01 44 27 60 43. </em>
  </p>
{% endif %}

{% include "emails/_footer.html" %}

</body>
</html>
